# Backend

Backends manage data and process interactions.  They consist of two main methods:

* `registerView(view, postprocess)`: view's `render()` will be called whenever the
  backend is executed and its result changes
* `execute(q)`: process a new query and emit its result as a `"data"` event. 
  Typically, a query is an array of objects, but its interpretation is defined
  by the backend.  For instance the IdentityBackend just returns its query argument,
  the difftree backend expects a difftree spec, and the duckdb backend expects
  a query string.


## JSON Spec

See the following typescript interface definitions: 

* [Backend Spec Interface](./spec.ts)
* [DiffTree Spec Interface](./dtspec.ts)


## Examples

The [ArrayBackend](./array.ts) is initialized to the `data` property.
`execute()` simply returns the initialized array.

```json
{
  "id": "d1",
  "type": "ArrayBackend",
  "data": [
    { "x": 1, "y": 2 },
    { "x": 3, "y": 4 }
  ]
}
```

The [IdentityBackend](./identity.ts) returns `execute(q)`'s query argument.

```json
{
  "id": "d5",
  "type": "IdentityBackend"
} 
```


The [DuckDBBackend](./duckdb.ts) currently initializes a single table.  The spec specifies
a URL to fetch the CSv and the tablename `"cars"`

```json
{
  "id": "duckdb1",
  "type": "DuckDBBackend",
  "name": "cars",
  "url": "/src/assets/cars.csv",
}
``` 

The [ClientDiffTreeBackend](./clientdt.ts) wraps the backend with id `"duckdb1"`
and initializes a difftree with the `difftree` spec.  The difftree simply chooses
between `"1"` and `"2"`.

```json
{
  "id": "dt1",
  "type": "ClientDiffTreeBackend",
  "backend": "duckdb1",
  "difftree": {
    "id": "n754",
    "type": "any",
    "children": [
      {
        "id": "n115",
        "type": "literal",
        "value": "1"
      },
      {
        "id": "n118",
        "type": "literal",
        "value": "2"
      }
    ]
  }
}
``` 

# DiffTree representation and binding

The current difftree spec and binding procedure is fairly straightforward.  You can read the short code blocks:

* [DiffTree spec](./dtspec.ts)
* [DiffTree binding](./dt.ts)

**Challenges**

Unfortunately, we have several points of ambiguity and limitations.
The goal here is to make the semantics of binding general and well-defined.
To do so, let's go over current binding semantics.


## Current Binding Semantics

Let us consider the following difftree to highlight how things currently work
and our limitations.  Each number is the choice node's ID.
For instance, `ANY:1` has id 1, and has three children: nodes 2, 3, 5.   

```
    ANY:1
     |----ANY:2
     |----OPT:3
     |      |---ANY:4
     |           |----Literal(10)
     |           |----Literal(200)
     |
     |----MULTI:5
            |---ANY:6
                 |----ANY:7
```

Although the tree is hierarchical, the binding is over a flattened version of the tree.
All nodes with cardinality one can be flattened into the same level, while
nodes with >1 cardinality cannot.  Thus the binding for the above would have the following
structure:   Each key is the ID of the corresponding choice node.

```ts
let binding = {
  1:, 2:, 3:, 4:,
  5: [
   { 6:, 7: }
   ...
  ]
}
```

Here, nodes 1-5 are at the top level, `MULTI:5` expects an Array of flattened bindings.
This structure is nice for several reasons:

* In most cases, a widget can directly set the global bindings.  For instance, if a Dropdown is bound to `ANY:2`, it can directly set `binding[2]`.    
* The logic for applying bindings is very simple.   
  * Each choice node reads the appropriate choice from the binding object.   
  * The multi-node binds its child with each element in its array. 
* The logic supports nested MULTI nodes


<a name="multi-hard"></a>
### MULTI bindings are tricky

Generating bindings for `MULTI:5` can be tricky.  There are two main cases:

1. A visualization interaction like MULTI or multi-click is bound to `MULTI:5`.  Since the interaction naturally generates a table of rows, we simply need to know which attributes should be bound to nodes 6 and 7.   See [the interaction README](../interact/README.md) for examples of `spec.h` that handle this case.
2. A widget like an Adder is bound to `MULTI:5`.  An adder adds/removes components to the interface -- for instance, clicking add will add a radio list bound to `ANY:6` and a slider bound to `ANY:7`.   Let's say the user has clicked twice, so the interface looks like:
I've listed the binding for each pair of radio/slider below the component.

```
[Adder] -> MULTI:5

    [ (x) apple ( ) orange ] -> ANY: 6 
    [ ------O------------- ] -> ANY: 7 
    binding: { 6: 0, 7: 30 }


    [ ( ) apple (x) orange ] -> ANY: 6 
    [ ----------------O--- ] -> ANY: 7 
    binding: { 6: 1, 7: 80 }
```

The challenge is that each pair of radio/sliders cannot directly edit
the global bindings!   Instead, they need to send their bindings to `MULTI:5`
so that it can construct the appropriate bindings:
```
  {
    5: [
      { 6: 0, 7: 30 }
      { 6: 1, 7: 80 }
    ]
  }
```

What does this mean?

* Supporting condition 2 (Adder-type widgets) requires communicating binding information up the widget hierarchy.
* Supporting _nested_ MULTI nodes can also be tricky.


<a name="problem-any"></a> 
### ANY bindings are ambiguous

There is a lot of magic in how [ANY bindings are handled](./dt.ts).
This stems from two separate issues:

1. bindings are often strings instead of numbers
2. ANY is sometimes implicitly treated as a VAL node


Widgets often generate strings rather than numbers, so the ANY node needs to do type coersion.  But the coersion is a bit sloppy.  It checks if the string can be cast to a number, if not the ANY treats itself as a VAL node and returns the string.   Otherwise it treats the coerced number as an index _unless_ the index is out of range, in which case it treats it as a string again.

There are a couple options to solve this:

1. extend the interaction `spec.h` property to specify type info.
  The interaction is responsible for all type casting, so the difftree can expect "clean" bindings.  

2. explicitly distinguish ANY nodes that index into a fixed set of options from VAL nodes that "pass its bindings through". 

### OPT/nested Choice Nodes are Implicit

Currently an `OPT` node is responsible for checking whether any of its descendents have been chosen, and if so, to set itself.   I don't think this is logic that the difftree should be responsible for!   The widgets should be responsible for generating correct bindings, since it needs to ensure visual consistency anyways.



