// 
// This file contains helper methods for working with PI-related specs
// including spec rewriting, interpreting interaction specs, and
// PI-specific type coersion issues.
//
import * as R from "ramda";
//
// Rewrite all spec conveniences into standard spec form.
//
// This is to ensure compatibility between programmatic Component
// creation and the spec PI2 generates
//
export function rewriteSpec(spec) {
    //
    // rewrite view.data properties into ArrayBackends
    //
    let newBackends = (spec.views ?? []).map(createViewBackend);
    newBackends = R.reject(R.isNil, newBackends);
    let backends = R.concat(newBackends, spec.backends);
    spec.backends = backends;
    //delete viewSpec.data;
    // 
    // rewrite interaction.h into a postprocessing function
    //
    (spec.interactions ?? []).forEach((iSpec) => {
        iSpec.postprocess = createInteractionPostprocessFunction(iSpec);
    });
    return spec;
}
function createViewBackend(viewSpec) {
    if (viewSpec.backend || !viewSpec.data)
        return null;
    if (R.isNil(viewSpec.id))
        console.error("view spec must contain an ID", viewSpec);
    let backendId = `viewbackend-${viewSpec.id}`;
    viewSpec.backend = backendId;
    let data = viewSpec.data;
    if (!R.is(Array, data))
        data = [data];
    return {
        id: backendId,
        data,
        type: "ArrayBackend"
    };
}
// transforms h JSON spec into an interaction postprocess function
//
// @param spec interaction spec
//
export function createInteractionPostprocessFunction(spec) {
    let funcs = [
        //R.partial(resolveData, [spec.t]), 
        R.partial(apply, [spec.h])
    ];
    return R.pipe(...funcs);
}
// Apply h specification to interaction state
//
// TODO: replace with https://ramdajs.com/docs/#evolve
function apply(h, state) {
    const nkeys = R.keys(state).length;
    // by default, should be identity function
    if (R.isNil(h) || R.keys(h).length === 0)
        return state;
    let nestedState = null;
    if (h["*"]) {
        nestedState = getNestedState(h["*"], nkeys, state);
        const nestedH = R.omit("*", h);
        return nestedState.map((row) => apply(nestedH, row));
    }
    let ret = {};
    for (const key in h) {
        const v = h[key];
        if (R.is(Function, v))
            ret[key] = v(state);
        else if (R.is(String, v))
            ret[key] = R.path(v.split("."), state);
        else if (R.is(Object, v))
            ret[key] = apply(v, state);
        else
            ret[key] = v;
    }
    return ret;
}
// promotes state's attribute into an array
function getNestedState(star, nkeys, state) {
    let nestedState = null;
    // if there's only 1 key in the state, use its value
    // otherwise, retrieve state[attrpath]
    if (star === "*" && nkeys == 1)
        nestedState = R.values(state)[0] ?? [];
    else
        nestedState = R.pathOr([], star.split("."), state);
    // wrap result as array
    if (!R.is(Array, nestedState))
        nestedState = [nestedState];
    return nestedState;
}
let zp = (num, places) => String(num).padStart(places, '0');
let dateToString = (date) => `${zp(row.getYear() + 1900, 4)}-${zp(row.getMonth() + 1, 2)}-${zp(row.getDate(), 2)}`;
let getBounds = (data) => {
    let reducer = ([min, max], v) => [Math.min(min, v), Math.max(max, v)];
    return R.reduce(reducer, data, [Number.MAX_VALUE, Number.MIN_VALUE]);
};
let dataToXBounds = (data) => {
    let [min, max] = getBounds(data);
    return [{ xmin: min.toString(), xmax: max.toString(), selected: 1 }];
};
let dataToYBounds = (data) => {
    let [min, max] = getBounds(data);
    return [{ ymin: min.toString(), ymax: max.toString(), selected: 1 }];
};
let dataToXYBounds = (data) => {
    return [{
            xmin: Math.min(data[0], data[1]).toString(),
            xmax: Math.max(data[0], data[1]).toString(),
            ymin: Math.min(data[2], data[3]).toString(),
            ymax: Math.max(data[2], data[3]).toString(),
            selected: 1
        }];
};
// XXX: there should be a more general/less hard-coded
// way to post-process manipulation state
//
// @param typ  interaction.m.type
// @param data interaction state
function resolveData(typ, data) {
    if (data.length === 0)
        return [{
                selected: 0,
                xmin: '-inf',
                xmax: 'inf',
                ymin: '-inf',
                ymax: 'inf'
            }];
    // deal with dates??
    data = data.map((row) => (row instanceof Date) ? dateToString(row) : row);
    // special case hacks for vis range interactions
    if (["BRUSHX", "ZOOMX", "PANX"].includes(typ))
        return dataToXBounds(data);
    if (["BRUSHY", "ZOOMY", "PANY"].includes(typ))
        return dataToYBounds(data);
    if (["BRUSHXY", "ZOOMXY", "PANXY"].includes(typ))
        return dataToXYBounds(data);
    return data;
}
