# Interaction

An interaction binds a source (widget or view) to a backend.  It also coerces the state generated by the source to a form expected by the backend.   

[interaction.ts](./interaction.ts) defines an `Interaction` class and a helper `InteractionLibrary` class.


The contract for an interaction is as follows

* The source view always generates a state with type `Array<object>`.    
* The target backend always receives a query with type `Array<object>`.
* The interaction transforms the state to a query.


Open Challenges


* A good definition for **m** to specify the source view's manipulation.  We need to decide on a simple language that is easy to specify and check within a View's implementation.
* A good definition for **h** to specify coercions using the JSON spec.  Should be general enough to be useful, but not too complex, and should make all decisions explicit.  The current design is specialized to our current implementation of DiffTrees.

Proposals

* [Proposal for changing the definition for **h**](../../Proposals.md#proposal-h)


## JSON Spec

The spec is defined in the [Typescript interface definition](./spec.ts).


## Example 

Let's consider interaction specs 0 and 1 in turn.

```ts

{
  "interactions": [
    {
      "id": 0,
      "source": "dropdown1",
      "target": 1,   // difftree
      "h": {
        "cn1": "value"
      }
    },
    {
      "id": 1,
      "source": 0, // a scatterplot
      "target": 1, // difftree 
      "m": {
        type: "MULTICLICK",
        space: "mark"
      }
      "h": {
          "cn1": "x",
          "cn2": "y"
          "cn3": "*"
      }
    }
  ]
}
```

### Interaction 0

This interaction binds the state of a dropdown to a choice node with id `"cn1"` in the difftree with backend id 1.
The dropdown generates a single state row, such as

```ts
[{ index: 0, value: 10, label: "apples" }]
```

The `h` transform will generate the binding:

```ts
[{ cn1: 10 }]
```

The binding will be passed to the difftree, which will generate a SQL query, execute the query, and send the results to the  appropriate views.

```ts
backend.execute([{ cn1: 10 }])
```

### Interaction 1

The main difference in this example is that the manipulation is a `MULTICLICK`, which generates multiple state rows.  For instance:

```ts
[{x:1, y:2}, {x:3, y:4}]
```    

Notice that the *h* prop maps `"cn3": "*"`.   

```ts
"h" = {
  "cn1": "x",
  "cn2": "y"
  "cn3": "*"
}
```

Applying *h* would return the binding:

```ts
binding = {
  cn3: [
    { cn1: 1, cn2: 2 },
    { cn1: 3, cn2: 4 }
  ]
}
```

Basically, `"cn3"` maps to a list, and each row in the state is then mapped to the appropriate choice node ids.   This format simplifies the logic for handling difftree bindings.  



