import * as R from "ramda";
import { vl } from "./vega";
export var CARDINALITY;
(function (CARDINALITY) {
    CARDINALITY["ONE"] = "1";
    CARDINALITY["N"] = "N";
    CARDINALITY["MULTI"] = "*";
})(CARDINALITY || (CARDINALITY = {}));
export function triggerIacts(iacts, id, state) {
    let o = {};
    o[id] = state;
    iacts.forEach((iact) => iact.trigger(o));
}
// workaround since vl is null during module import,
// but populated when the application initializes
let vlFields = null;
function initVLFields() {
    if (!vlFields)
        vlFields = {
            "x": vl.x,
            "y": vl.y,
            "color": vl.color,
            "key": vl.key
        };
    return vlFields;
}
//
// Add the appropriate encodings to the mark object
//
// @param mark vega lite API's mark object
// @param schema visual encodings and data types supported by the vis
// @param labels mapping from visual encodings used in the plot to its label name
//
export function initializeMarkEncs(mark, schema, labels) {
    initVLFields();
    R.keys(labels).forEach((key) => {
        const label = labels[key] ?? key;
        const typ = schema[key]["type"];
        let enc = null;
        if (typ === "ordinal")
            enc = vlFields[key]().fieldN(key).title(label);
        else if (typ === "date")
            enc = vlFields[key]().fieldT(key).title(label);
        else
            enc = vlFields[key]().fieldQ(key).title(label).scale({ zero: false });
        mark = mark.encode(enc);
    });
    return mark;
}
export function fixDataTypes(data, schema, labels) {
    return data.map((row, i) => {
        for (const k in row) {
            if (schema[k].type === "quantitative") {
                if (labels[k] !== "date")
                    row[k] = parseFloat(row[k]);
            }
            else if (schema[k].type === "geojson") {
                row = R.mergeRight(JSON.parse(row[k]), R.omit(["geography"], row));
            }
            if (k === "color") {
                let num = parseInt(row[k]);
                if (!isNaN(num))
                    row[k] = num;
            }
        }
        return row;
    });
}
// @param typ type of the manipulation
// @returns vegalite interaction spec object
export function manipulationToVLSpec(typ) {
    switch (typ) {
        case "SINGLE":
            return vl.selectSingle().nearest(true);
        case "MULTI":
            return vl.selectMulti().toggle(false).nearest(true);
        case "BRUSHX":
            return vl.selectInterval().encodings(['x']);
        case "BRUSHY":
            return vl.selectInterval().encodings(['y']);
        case "BRUSHXY":
            return vl.selectInterval().encodings(['x', 'y']);
        case "PANX":
            return vl.selectInterval().bind("scales").encodings(['x']).zoom(false);
        case "PANY":
            return vl.selectInterval().bind("scales").encodings(['y']).zoom(false);
        case "PANXY":
            return vl.selectInterval().bind("scales").encodings(['x', 'y']).zoom(false);
        case "ZOOMX":
            return vl.selectInterval().bind("scales").encodings(['x']).zoom(true);
        case "ZOOMY":
            return vl.selectInterval().bind("scales").encodings(['y']).zoom(true);
        case "ZOOMXY":
            return vl.selectInterval().bind("scales").encodings(['x', 'y']).zoom(true);
        default:
            alert("interaction not supported " + typ);
    }
}
function singleHandler(iact, vegaview, event, item) {
    if (!item.datum)
        return null;
    let data = [item.datum.datum];
    if ("key" in data[0])
        data[0]["key"] = parseInt(data[0]["key"]).toString();
    for (var x in data[0])
        data[0][x] = data[0][x].toString();
    return data;
}
function multiHandler(iact, vegaview, event, item) {
    let data = vegaview.data(iact.sel.name + "_store");
    let viewdata = vegaview.data("source_0");
    if (data.length == 0)
        return null;
    let real_data = [];
    return data.map((d, i) => {
        let dd = viewdata[data[i].values[0] - 1];
        if ("key" in dd)
            dd["key"] = parseInt(dd["key"]).toString();
        for (var x in dd)
            dd[x] = dd[x].toString();
    });
}
function brushZoomPanHandler(iact, vegaview, event, item) {
    let data = vegaview.data(iact.sel.name + "_store");
    if (!data || data.length == 0)
        return [];
    if (["BRUSHXY", "ZOOMXY", "PANXY"].includes(iact.type)) {
        data = data[0].values[0].concat(data[0].values[1]);
    }
    else {
        data = data[0].values[0];
    }
    return data;
}
//
// add listeners that coerce vega-specific event data
// into a form that we expect
//
// XXX: I'm completely confused how brush/zoom/pan manipulations
//      are listened to.  I couldn't find any listeners in the pi-client 
//      source for Point other than click and wheel (below), but a brushing 
//      interaction is not a click, so it's not clear to me why logic to handle
//      brush/pan/zoom interactions are even in the click event handler.
//
export function addVegaListeners(iact, vlview, cb) {
    if (!cb)
        return;
    let vegaview = vlview.view;
    vegaview.addDataListener("source_0", (event, item) => { });
    vegaview.addEventListener('click', (event, item) => {
        let data = null;
        if (iact.type == "SINGLE")
            data = singleHandler(iact, vegaview, event, item);
        else if (iact.type == "MULTI")
            data = multiHandler(iact, vegaview, event, item);
        else
            data = brushZoomPanHandler(iact, vegaview, event, item);
        if (data)
            cb(iact.type, data);
    });
    vegaview.addEventListener('wheel', (event, item) => {
        let data = null;
        if (["SINGLE", "MULTI"].includes(iact.type)) {
            if (item.datum != null)
                data = [item.datum];
        }
        else {
            data = brushZoomPanHandler(iact, vegaview, event, item);
            data = vegaview.data(iact.sel.name + "_store");
            if (!data.length)
                data = null;
        }
        if (data)
            cb(iact.type, data);
    });
}
// Post-process the spec for formatting and customization reasons
export let vlApiToSpec = (vlobj, data) => {
    let vlspec = vlobj.toSpec();
    vlspec.axis = {
        titleFontSize: 15, labelFontSize: 15,
        labelOverlap: "parity",
    };
    for (let key in vlspec.encoding) {
        vlspec.encoding[key].axis = {
            titleFontSize: 15, labelFontSize: 15,
            // https://github.com/vega/vega/issues/1019#issuecomment-333033862
            labelOverlap: "parity",
            ...(key === "x" && { labelAngle: 0 })
        };
    }
    // add spacing to quantitative positional attributes
    let enc = vlspec.encoding;
    if (enc && enc.x && enc.x.scale && enc.x.type === "quantitative") {
        enc.x.scale.padding = 15;
        enc.x.scale.nice = true;
    }
    if (enc && enc.y && enc.y.scale && enc.y.type === "quantitative") {
        enc.y.scale.padding = 15;
        enc.y.scale.nice = true;
    }
    if (data)
        vlspec.data = { name: "source_0", values: data };
    return vlspec;
};
